#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#define RAND_UNIFORME (double)random()/(double)RAND_MAX

double random_exp(double landa)
{
return (-log(1.0-RAND_UNIFORME)/(double)landa);
};


double random_normal(double esp,double var)
{
double u1=RAND_UNIFORME;
double u2=RAND_UNIFORME;
double z=sqrt(-2.0*log(u1))*cos(2*M_PI*u2);
return esp+var*z;
};

unsigned long random_poisson(double landa)
{
double p=exp(-landa);
double g=p;
double u=RAND_UNIFORME;
unsigned long k=0;
while (u>g)
    {
    p*=(landa/(double)(++k));
    g+=p;
    };
return k;
};

unsigned long random_binomial(unsigned long n, double p)
{
double t=p/(1-p);
double u=RAND_UNIFORME;
double p0=pow((1-p),n);
double g=p0;
unsigned int k=0;
while (u>g)
    {
    p0*=t*(n-k)/(k+1);
    g+=p0;
    k++;
    };
return k;
};

long double fact(unsigned long x,unsigned long f)
{
if (x==f) return 1.0;
if (x<f) return 1.0/fact(f,x);
long double t=1.0;
while (x>f)
    {
    t*=x;
    x--;
    };
return (long double)t;
};

double hipergeom_p0(unsigned long N,unsigned long D,unsigned long n)
{
long k=n-N+D;
if (k<0) k=0;
return (double)((fact(D,D-k)*fact(N-D,N-D-n+k)*fact(n,n-k))/(fact(k,1)*fact(N,N-n)));

//return ((double)(fact(D)/fact(D-k))*(fact(N-D)/fact(N-D-n+k))*(fact(n)/fact(n-k)))/(double)(fact(k)*(fact(N)/fact(N-n)));
};

long random_hipergeom(unsigned long N,unsigned long D,
						unsigned long n,double p=0.0)
{
double u=RAND_UNIFORME;
long k=n-N+D;
if (k<0) k=0;
if (p==0.0) p=hipergeom_p0(N,D,n);
double g=p;
while (u>g)
    {
    p*=((double)(D-k)*(n-k))/(double)((k+1)*(N-D-n+k+1));
    g+=p;
    k++;
    };
return k;
};

int main(int argc, char **argv)
{
if (argc<3) 
   {
   printf("ProbabLib by Antonio Larrosa Jimenez . Malaga (Spain) 1998\n");
   printf("probablib <numbers to generate> <exp|normal|poisson|binomial|hipgeom|binomneg> <options>\n");
   printf("if exp     : options = <lambda>\n");
   printf("if norm    : options = <mean> <var>\n");
   printf("if poisson : options = <lambda>\n");
   printf("if binomial: options = <n> <p>\n");
   printf("if hipgeom : options = <N> <D> <n>\n");
 
   return -1;
   };
double r=atoi(argv[1]);
if (strcmp(argv[2],"exp")==0)
    {
    double landa=strtod(argv[3],NULL);
    while (r>0) 
	{
	printf("%g\n",random_exp(landa));
	r--;
	};
    }
 else if (strcmp(argv[2],"poisson")==0)
    {
    double landa=strtod(argv[3],NULL);
    while (r>0) 
	{
	printf("%ld\n",random_poisson(landa));
	r--;
	};
    }
 else if (strcmp(argv[2],"normal")==0)
    {
    double esp=strtod(argv[3],NULL);
    double var=strtod(argv[4],NULL);
    while (r>0) 
	{
	printf("%g\n",random_normal(esp,var));
	r--;
	};
    
    }
 else if (strcmp(argv[2],"binomial")==0)
    {
    unsigned long n=strtoul(argv[3],NULL,10);
    double p=strtod(argv[4],NULL);
    while (r>0) 
	{
	printf("%ld\n",random_binomial(n,p));
	r--;
	};
    
    }
 else if (strcmp(argv[2],"hipgeom")==0)
    {
    unsigned long N=strtoul(argv[3],NULL,10);
    unsigned long D=strtoul(argv[4],NULL,10);
    unsigned long n=strtoul(argv[5],NULL,10);
   double p=hipergeom_p0(N,D,n);
   while (r>0) 
	{
	printf("%ld\n",random_hipergeom(N,D,n,p));
	r--;
	};
    
    };
return 0;
};
